#!/usr/bin/env python
# encoding: utf-8

def get_parameter(args, configuration, keyword, default):
  """ Get the right value for a parameter

  The parameters are either defined in a separate configuration file
  or given directly via command-line. Note that the command-line
  has priority over the configuration file.

  As a convention, parameters made with more than one word (i.e. batch size)
  are provided with an underscore in the configuration file, and with an
  hyphen in the command-line:

    - configuration:  batch_size=64
    - command line:   --batch-size=64

  Parameters
  ----------
  args: dictionary
    The arguments as parsed from the command line.
  configuration: object
    The arguments given by the configuration file.
  keyword: string
    the keyword for the parameter to process (in the "configuration" style)
  default: 
    The default value of the parameter

  Returns
  -------
  arg:
    The right value for the given keyword argument

  """
 
 # get the keyword in a "docopt" friendly format
  args_kw = '--' + keyword.replace('_', '-')
  
  # get the type of this argument
  _type = type(default)

  # get the default value 
  arg_default = default 

  # get the argument in the configuration file 
  if hasattr(configuration, keyword):
    arg_config = getattr(configuration, keyword)

  # get the argument from the command-line
  arg_command = _type(args[args_kw])
 
  # if the argument was not specified in the config file
  if not hasattr(configuration, keyword):
    return arg_command
  else:
    if (arg_command == arg_default):
      return arg_config
    else:
      return arg_command

